import time
import hall_sensor
import pressure_sensor
import parameters
from alarm import Alarm


class StateMachine:

    def __init__(self) -> None:
        self.previous_pressure = pressure_sensor.get_pressure()
        time.sleep(0.02)
        self.__measure_pressure()
        
    def start_task_loop(self):
        while True:
            self.wait_for_correct_rotation_frequency()
            self.handle_pressure()
            time.sleep(0.03)

    def wait_for_correct_rotation_frequency(self):
        while not self.__rotation_is_within_limits():
            time.sleep(0.03)

    def __measure_pressure(self):
        self.current_pressure = pressure_sensor.get_pressure()

    def handle_pressure(self):
        self.__measure_pressure()
        is_saturated = self.__pressure_is_higher_than_maximum_saturation()
        if is_saturated:
            Alarm.trigger_saturated_alarm()
        else:
            self.scan_holes_in_filter()

    def scan_holes_in_filter(self):
        there_is_a_hole = self.__pressure_is_lower_than_before()
        if there_is_a_hole:
            Alarm.trigger_hole_alarm()

    def __pressure_is_lower_than_before(self) -> bool:
        pressure_is_lower = (
            self.current_pressure <
            self.previous_pressure * (1 - parameters.PRESSURE_TOLERANCE_PERCENTAGE / 100)  
        )
        return pressure_is_lower

    def __pressure_is_higher_than_maximum_saturation(self) -> bool:
        pressure_is_not_lower = (
            self.current_pressure > parameters.PRESSURE_SATURATION_THRESHOLD  
        )
        return pressure_is_not_lower

    def __rotation_is_within_limits(self) -> bool:
        rotation = hall_sensor.get_rotation_frequency()
        is_within_limits = parameters.REFERENCE_ROTATION_DOWN_LIMIT < rotation < parameters.REFERENCE_ROTATION_UPPER_LIMIT 
        return is_within_limits
